

# DIO02 - OCTAL DIGITAL I/O DEVICE 1.1.0


# Imports
from ctypes import *
from . import Device
from . import Error


# Class DIO02
class DIO02(Device):

    # Channels
    CHANNELS = 8

    # Directions
    DIRECTION_INPUT = False
    DIRECTION_OUTPUT = True

    # States
    STATE_LOW = False
    STATE_HIGH = True

    # Configure Library
    Device._lib.dio02_read.argtypes = [c_uint, POINTER(c_ubyte), c_ubyte, POINTER(c_ubyte), POINTER(c_ubyte)]
    Device._lib.dio02_read.restype = c_ubyte
    Device._lib.dio02_set_direction.argtypes = [c_uint, POINTER(c_ubyte), c_ubyte, POINTER(c_ubyte), c_ubyte]
    Device._lib.dio02_set_direction.restype = c_ubyte
    Device._lib.dio02_write.argtypes = [c_uint, POINTER(c_ubyte), c_ubyte, POINTER(c_ubyte), c_ubyte]
    Device._lib.dio02_write.restype = c_ubyte


    # Set Direction
    def direction(self, *args):

        # Check Parameters
        if len(args) == 1:

            # Check Direction
            if isinstance(args[0], bool):

                # Set default Channel List and Direction List
                channel = self._array(c_ubyte, range(1, self.CHANNELS + 1))
                direction = self._array(c_ubyte, [args[0]] * self.CHANNELS);

            elif isinstance(args[0], dict):

                # Set Channel List and Direction List
                channel = self._array(c_ubyte, args[0].keys())
                direction = self._array(c_ubyte, args[0].values())

            else:

                # Error
                raise Error.Parameter

        elif len(args) == 2:

            # Check Channel
            if isinstance(args[0], int):

                # Set Channel and Direction List
                channel = self._array(c_ubyte, [args[0]])
                direction = self._array(c_ubyte, [args[1]])

            elif isinstance(args[0], set):

                # Set Channel and Direction List
                channel = self._array(c_ubyte, args[0])
                direction = self._array(c_ubyte, [args[1]] * len(args[0]))

            else:

                # Error
                raise Error.Parameter

        else:

            # Error
            raise Error.Parameter

        # Set Direction
        error = self._lib.dio02_set_direction(self._address, channel, self._value(c_ubyte, len(channel)), direction,
            self._value(c_ubyte, len(direction)))

        # Error Handler
        self._error(error)


    # Read
    def read(self, *args):

        # Check Parameters
        if len(args) == 0:

            # Set default Channel List
            channel = self._array(c_ubyte, range(1, self.CHANNELS + 1))

        elif len(args) == 1:

            # Check Channel
            if isinstance(args[0], int):

                # Set Channel List
                channel = self._array(c_ubyte, [args[0]])

            elif isinstance(args[0], set):

                # Set Channel List
                channel = self._array(c_ubyte, args[0])

            else:

                # Error
                raise Error.Parameter

        else:

            # Error
            raise Error.Parameter

        # State List
        state = self._array(c_ubyte, len(channel))

        # Read
        error = self._lib.dio02_read(self._address, channel, self._value(c_ubyte, len(channel)), state,
            self._value(c_ubyte, len(state)))

        # State List
        state_ = {}

        # Parse State List
        for i in range(len(channel)):

            # Insert State into List
            state_[channel[i]] = state[i]

        # Return State List
        return state_ if ((len(args) == 0) or isinstance(args[0], set)) else state_[channel[0]]


    # Write
    def write(self, *args):

        # Check Parameters
        if len(args) == 1:

            # Check State
            if isinstance(args[0], dict):

                # Set Channel List and State List
                channel = self._array(c_ubyte, args[0].keys())
                state = self._array(c_ubyte, args[0].values())

            else:

                # Set default Channel List and State List
                channel = self._array(c_ubyte, range(1, self.CHANNELS + 1))
                state = self._array(c_ubyte, [args[0]] * self.CHANNELS);

        elif len(args) == 2:

            # Check Channel
            if isinstance(args[0], int):

                # Set Channel and State List
                channel = self._array(c_ubyte, [args[0]])
                state = self._array(c_ubyte, [args[1]])

            elif isinstance(args[0], set):

                # Set Channel and State List
                channel = self._array(c_ubyte, args[0])
                state = self._array(c_ubyte, [args[1]] * len(args[0]))

            else:

                # Error
                raise Error.Parameter

        else:

            # Error
            raise Error.Parameter

        # Write
        error = self._lib.dio02_write(self._address, channel, self._value(c_ubyte, len(channel)), state,
            self._value(c_ubyte, len(state)))

        # Error Handler
        self._error(error)
