

# VI01 - OCTAL VOLTAGE MONITOR 1.1.0


# Imports
from ctypes import *
from . import Device
from . import Error


# Class VI01
class VI01(Device):

    # Channels
    CHANNELS = 8

    # Sampling Frequencies [Hz]
    FREQUENCY_6 = 6
    FREQUENCY_12 = 12
    FREQUENCY_25 = 25
    FREQUENCY_50 = 50
    FREQUENCY_100 = 100
    FREQUENCY_200 = 200
    FREQUENCY_400 = 400
    FREQUENCY_800 = 800
    FREQUENCY_1500 = 1500
    FREQUENCY_3000 = 3000

    # Configure Interface
    Device._lib.vi01_measure.argtypes = [c_uint, POINTER(c_ubyte), c_ubyte, c_uint, c_float, POINTER(c_float),
        POINTER(c_uint)]
    Device._lib.vi01_measure.restype = c_ubyte
    Device._lib.vi01_set_frequency.argtypes = [c_uint, c_ushort]
    Device._lib.vi01_set_frequency.restype = c_ubyte
    Device._lib.vi01_stop.argtypes = [c_uint]
    Device._lib.vi01_stop.restype = c_ubyte


    # Measure [V]
    def measure(self, *args):

        # Check Parameters
        if len(args) == 0:

            # Set default Channel List
            channel = self._array(c_ubyte, range(1, self.CHANNELS + 1))

        else:

            # Check Channel
            if isinstance(args[0], int):

                # Set Channel List
                channel = self._array(c_ubyte, [args[0]])

            elif isinstance(args[0], set):

                # Set Channel List
                channel = self._array(c_ubyte, args[0])

            else:

                # Error
                raise Error.Parameter

        # Check Parameters
        if (len(args) == 0) or (len(args) == 1):

            # Set default Samples and Frequency
            samples = self._value(c_uint, 1)
            frequency = self._value(c_float)

        elif len(args) == 3:

            # Set Samples and Frequency
            samples = self._value(c_uint, args[1])
            frequency = self._value(c_float, args[2])

        else:

            # Error
            raise Error.Parameter

        # Voltage List
        voltage = self._array(c_float, len(channel) * samples.value)

        # Measure Voltage
        error = self._lib.vi01_measure(self._address, channel, self._value(c_ubyte, len(channel)), samples, frequency,
            voltage, self._value(c_uint, len(voltage)))

        # Error Handler
        self._error(error)

        # Sample List
        sample = []

        # Parse Sample List
        i = 0
        for j in range(samples.value):

            # Check Parameters and Channel
            if (len(args) == 0) or (isinstance(args[0], set)):

                # Sample
                sample_ = {}

                # Parse Channel List
                for channel_ in channel:

                    # Set Sample
                    sample_[channel_] = voltage[i]
                    i += 1

            else:

                # Set Sample
                sample_ = voltage[i]
                i += 1

            # Insert Sample into List
            sample.append(sample_)

        # Return Sample List
        return (sample if (len(args) == 3) else sample[0])


    # Stop
    def stop(self):

        # Stop
        error = self._lib.vi01_stop(self._address)

        # Error Handler
        self._error(error)


    # Set Frequency [Hz]
    def frequency(self, frequency):

        # Set Frequency
        error = self._lib.vi01_set_frequency(self._address, self._value(c_ushort, frequency))

        # Error Handler
        self._error(error)
